package com.hhwy.qbeqsjy.common;

import java.io.*;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.*;


/**
 * TODO desc
 *
 * @ClassName: CommonUtil
 * @author: hwang
 * @date: 2021/3/30 14:48
 */
public class CommonUtil {

  private static ResourceBundle bundle;

  static {
    bundle = ResourceBundle.getBundle("application");
  }

  /**
   * 长整型IP转换成正常IP
   * @param num
   * @return
   */
  public static String intToIp(Long num){
    if (num < 0L){
      num += 4294967296L;
    }
    StringBuilder sb = new StringBuilder(15);
    sb.append(num >> 24);
    sb.append('.');
    sb.append((num & 16777215L) >> 16);
    sb.append('.');
    sb.append((num & 65535L) >> 8);
    sb.append('.');
    sb.append(num & 255L);
    return sb.toString();
  }

  /**
   * 字符串判空
   * @param str
   * @return
   */
  public static boolean isNullOrEmpty(String str)
  {
    if (str == null || "null".equals(str.toLowerCase().trim()) || "[null]".equals(str.toLowerCase().trim()) || "[]".equals(str.toLowerCase().trim()))
    {
      return true;
    }

    return str.trim().isEmpty();
  }


  /**
   * 经纬度计算距离  单位(米)
   * @param lng1
   * @param lat1
   * @param lng2
   * @param lat2
   * @return
   */
  /*public static double getDistance2(Double lng1, Double lat1, Double lng2, Double lat2){
    double earth = 6371.393;
    double radLat1 = rad(lat1);
    double radLat2 = rad(lat2);
    double a = radLat1 - radLat2;
    double b = rad(lng1) - rad(lng2);
    double s = 2* Math.asin(Math.sqrt(Math.pow(Math.sin(a/2),2)+Math.cos(radLat1)*Math.cos(radLat2)*Math.pow(Math.sin(b/2),2)));
    s=s*earth;
    s=Math.round(s*10000d) / 10000d;
    s=s*1000;
    return s;
  }*/

  /*private static double rad(double d){
    return d * Math.PI / 180.0;
  }*/


  /**
   * 递归加载输入文件
   * @param filePath
   * @param filelist
   * @param preStr  文件前缀  contains(preStr)
   * @return
   */
  public static List<String> getFiles(String filePath, List<String> filelist ,String preStr) {
    File root = new File(filePath);
    if (root.exists()) {
      File[] files = root.listFiles();
      for (File file : files) {
        if (file.isDirectory()) {
          getFiles(file.getAbsolutePath(), filelist,preStr);
        } else {
          //文件前缀包含fzp的文件
          if (file.getName().contains(preStr)){
            filelist.add(file.getAbsolutePath());
          }
        }
      }
    }
    return filelist;
  }



  /**
   * 获取当前时间的yyyyMMdd时间格式
   * @return
   */
  public static String getyyyyMMdd(){
    SimpleDateFormat formatter = new SimpleDateFormat("yyyyMMdd");
    return formatter.format(System.currentTimeMillis());
  }


  /**
   * 获取当前时间的yyyyMMddHHmmss时间格式
   * @return
   */
  public static String getyyyyMMddHHmmss(){
    SimpleDateFormat formatter = new SimpleDateFormat("yyyyMMddHHmmss");
    return formatter.format(System.currentTimeMillis());
  }


  /**
   * 获取当前时间戳的yyyyMMdd时间格式
   * @return
   */
  public static String timestamp2yyyyMMdd(String timestamp){
    try {
      DateFormat formatter = new SimpleDateFormat("yyyyMMdd");
      long ms = Long.parseLong(timestamp);
      Date date = formatter.parse(formatter.format(ms));
      String dateStr = formatter.format(date);
      return dateStr;
    } catch (Exception e) {
      e.printStackTrace();
      return null;
    }
  }


  /**
   * 动态获取配置文件中指定 key的值
   * @return
   */
  public static String getConfValueByKey(String key){
    String value = "";
    try {                                       //避免中文乱码
      value = new String(bundle.getString(key).getBytes("ISO-8859-1"), "UTF-8");
    } catch (UnsupportedEncodingException e) {
      e.printStackTrace();
    }
    return value;
  }


  public static byte[] getTestFile(String filePath) throws IOException {
    InputStream in = new FileInputStream(filePath);
    byte[] data = toByteArray(in);
    in.close();
    return data;
  }


  public static byte[] toByteArray(InputStream in) throws IOException {
    ByteArrayOutputStream out = new ByteArrayOutputStream();
    byte[] buffer = new byte[1024 * 4];
    int n = 0;
    while ((n = in.read(buffer)) != -1) {
      out.write(buffer, 0, n);
    }
    return out.toByteArray();
  }


  /**
   * 根据 key移除重复的 map
   * @param list
   * @param mapKey
   * @return
   */
  public static List<Map<String, Object>> removeRepeatMapByKey(List<Map<String, Object>> list, String mapKey){
    List<Map<String, Object>> listMap = new ArrayList<>();
    if(list == null || list.size() == 0){
      return listMap;
    }
    Map<String, Object> msp = new HashMap<>();
    for (int i = list.size() - 1; i >= 0; i--) {
      Map<String, Object> map = list.get(i);
      String idno = (String) map.get(mapKey);
      map.remove(mapKey);
      msp.put(idno, map);
    }
    Set<String> mspKey = msp.keySet();
    for (String key : mspKey) {
      Map<String, Object> newMap = (Map<String, Object>) msp.get(key);
      newMap.put(mapKey, key);
      listMap.add(newMap);
    }
    return listMap;
  }


  /**
   * 线程sleep n秒
   *
   * @param millis 毫秒数
   */
  public static void threadSleep(long millis) {
    try {
      Thread.sleep(millis);
    } catch (Exception e) {
      e.printStackTrace();
    }
  }

}
