package com.hhwy.qbeqsjy.common;

import org.apache.commons.lang3.StringUtils;
import org.apache.http.HttpEntity;
import org.apache.http.HttpStatus;
import org.apache.http.NameValuePair;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.*;
import org.apache.http.entity.InputStreamEntity;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * http操作类 <功能详细描述>
 *
 * @author nzhou
 * @version [版本号, 2016年6月7日]
 * @see [相关类/方法]
 * @since [产品/模块版本]
 */
public class HttpClient {
    /**
     * 日志
     */
    private static final Logger LOG = LoggerFactory.getLogger(HttpClient.class);

    /**
     * socket超时
     */
    private static final int SOCKET_TIMEOUT = 180000;

    /**
     * 连接超时
     */
    private static final int CONN_TIMEOUT = 600000;

    private CloseableHttpClient httpClient;

    private RequestConfig requestConfig;

    /**
     * 构造客户端连接对象 <功能详细描述>
     *
     * @see [类、类#方法、类#成员]
     */
    public HttpClient() {
        try {
            this.httpClient = HttpClients.createDefault();
            this.requestConfig = RequestConfig.custom().setSocketTimeout(SOCKET_TIMEOUT)
                    .setConnectTimeout(CONN_TIMEOUT).setConnectionRequestTimeout(CONN_TIMEOUT).build();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 发送POST请求 <功能详细描述>
     *
     * @param url	路径
     * @param body	消息体
     * @return
     * @throws UnsupportedEncodingException
     * @see [类、类#方法、类#成员]
     */
    public String postResource(String url, String body, Map<String, Object> headerParamsMap) {
        HttpPost postMethod = new HttpPost(url);

        if(headerParamsMap != null && headerParamsMap.size() > 0){
            Set<String> keys = headerParamsMap.keySet();
            for (String key : keys) {
                postMethod.setHeader(key, headerParamsMap.get(key).toString());
            }
        }
        postMethod.setConfig(this.requestConfig);
        postMethod.setHeader("Content-Charset", "UTF-8");
        postMethod.setHeader("Accept-Charset", "UTF-8");
        postMethod.setHeader("Content-Type", "application/json");
        postMethod.setEntity(new StringEntity(body, "utf-8"));

        CloseableHttpResponse response = null;
        String content = null;
        try {
            response = this.httpClient.execute(postMethod);
            int httpStatus = response.getStatusLine().getStatusCode();
            if (httpStatus != HttpStatus.SC_OK) {
                // 返回的是失败响应
                LOG.error(
                        String.format("Failed to execute post method '%s', caused by: it's response statuscode is %d!",
                                url, httpStatus));
            }
            HttpEntity entity = response.getEntity();
            if (entity != null) {
                content = EntityUtils.toString(entity, "UTF-8");
                EntityUtils.consume(entity);
            }
        } catch (Exception e) {
            LOG.error("url：" + url + "请求出错！", e);
            e.printStackTrace();
        } finally {
            if (response != null) {
                try {
                    response.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            postMethod.releaseConnection();
        }
        return content;
    }


    public String postResourceTXWA(String url, String body, String token, String resourceUrl) {
        HttpPost postMethod = new HttpPost(url);
        postMethod.setConfig(this.requestConfig);
        postMethod.setHeader("Content-Charset", "UTF-8");
        postMethod.setHeader("Accept-Charset", "UTF-8");
        postMethod.setHeader("Content-Type", CommonUtil.getConfValueByKey("TXWA_REQUEST_Content-Type"));
        //添加token头信息
        if(StringUtils.isNotBlank(token)){
            postMethod.setHeader("token", token);
        }
        if(StringUtils.isNotBlank(resourceUrl)){
            postMethod.setHeader("resourceUrl", resourceUrl);
        }
        postMethod.setEntity(new StringEntity(body, "utf-8"));

        CloseableHttpResponse response = null;
        String content = null;
        try {
            response = this.httpClient.execute(postMethod);
            int httpStatus = response.getStatusLine().getStatusCode();
            if (httpStatus != HttpStatus.SC_OK) {
                // 返回的是失败响应
                LOG.error(
                        String.format("Failed to execute post method '%s', caused by: it's response statuscode is %d!",
                                url, httpStatus));
            }
            HttpEntity entity = response.getEntity();
            if (entity != null) {
                content = EntityUtils.toString(entity, "UTF-8");
                EntityUtils.consume(entity);
            }
        } catch (Exception e) {
            LOG.error("url：" + url + "请求出错！", e);
            e.printStackTrace();
        } finally {
            if (response != null) {
                try {
                    response.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            postMethod.releaseConnection();
        }
        return content;
    }


    /**
     * 发送POST请求 <功能详细描述>
     *
     * @param url	路径
     * @param body	消息体
     * @return
     * @throws UnsupportedEncodingException
     * @see [类、类#方法、类#成员]
     */
    public String postResourceXHJSON(String url, String body) {
        HttpPost postMethod = new HttpPost(url);

        postMethod.setConfig(this.requestConfig);
        postMethod.setHeader("Content-Charset", "UTF-8");
        postMethod.setHeader("Accept-Charset", "UTF-8");
        postMethod.setHeader("Content-Type", "application/json");
        postMethod.setHeader("requestType", CommonUtil.getConfValueByKey("XH_requestType"));
        postMethod.setHeader("serviceID", CommonUtil.getConfValueByKey("XH_serviceId"));
        postMethod.setHeader("secretKey", CommonUtil.getConfValueByKey("XH_secretKey"));
        postMethod.setEntity(new StringEntity(body, "utf-8"));

        CloseableHttpResponse response = null;
        String content = null;
        try {
            response = this.httpClient.execute(postMethod);
            int httpStatus = response.getStatusLine().getStatusCode();
            if (httpStatus != HttpStatus.SC_OK  || httpStatus != HttpStatus.SC_NO_CONTENT) {
                // 返回的是失败响应
                LOG.error(
                        String.format("Failed to execute post method '%s', caused by: it's response statuscode is %d!",
                                url, httpStatus));
            }
            HttpEntity entity = response.getEntity();
            if (entity != null) {
                content = EntityUtils.toString(entity, "UTF-8");
                EntityUtils.consume(entity);
            }
        } catch (Exception e) {
            LOG.error("url：" + url + "请求出错！", e);
            e.printStackTrace();
        } finally {
            if (response != null) {
                try {
                    response.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            postMethod.releaseConnection();
        }
        return content;
    }


    /**
     * 发送POST请求 <功能详细描述>
     *
     * @param url	路径
     * @param requestXML	消息体
     * @return
     * @throws UnsupportedEncodingException
     * @see [类、类#方法、类#成员]
     */
    public int postResourceXHxml(String url, String requestXML) {
        CloseableHttpClient httpClient = HttpClients.createDefault();
        CloseableHttpResponse response = null;
        int httpStatus = 500;
        try {
            InputStreamEntity reqEntity = new InputStreamEntity(new ByteArrayInputStream(requestXML.getBytes()));
            reqEntity.setContentType("application/xml");
            reqEntity.setContentEncoding("utf-8");
            reqEntity.setChunked(true);
            HttpUriRequest request = RequestBuilder.post()
                    .setUri(url)
                    .setEntity(reqEntity)
                    .setHeader("Content-Type", "application/xml")
                    .setHeader("requestType", CommonUtil.getConfValueByKey("XH_requestType"))
                    .setHeader("serviceID", CommonUtil.getConfValueByKey("XH_serviceId"))
                    .setHeader("secretKey", CommonUtil.getConfValueByKey("XH_secretKey"))
                    .build();
            LOG.info("请求星火服务注册接口，url：{}", url);
            response = httpClient.execute(request);
            httpStatus = response.getStatusLine().getStatusCode();
            LOG.info("响应码：{}", httpStatus);
        } catch (Exception e) {
            LOG.error("url：" + url + "请求出错！", e);
            e.printStackTrace();
        }finally {
            if (response != null) {
                try {
                    response.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            if (httpClient != null) {
                try {
                    httpClient.close();
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        }
        return httpStatus;
    }

    public int postResourceFirexml(String url, String requestXML) {
        CloseableHttpClient httpClient = HttpClients.createDefault();
        CloseableHttpResponse response = null;
        int httpStatus = 500;
        try {
            InputStreamEntity reqEntity = new InputStreamEntity(new ByteArrayInputStream(requestXML.getBytes()));
            reqEntity.setContentType("application/xml");
            reqEntity.setContentEncoding("utf-8");
            reqEntity.setChunked(true);
            HttpUriRequest request = RequestBuilder.post()
                    .setUri(url)
                    .setEntity(reqEntity)
                    .setHeader("Content-Type", "application/xml")
                    .setHeader("requestType", CommonUtil.getConfValueByKey("fire_requestType"))
                    .setHeader("serviceID", CommonUtil.getConfValueByKey("fire_serviceId"))
                    .setHeader("secretKey", CommonUtil.getConfValueByKey("fire_secretKey"))
                    .build();
            LOG.info("请求星火服务注册接口，url：{}", url);
            response = httpClient.execute(request);
            httpStatus = response.getStatusLine().getStatusCode();
            LOG.info("响应码：{}", httpStatus);
        } catch (Exception e) {
            LOG.error("url：" + url + "请求出错！", e);
            e.printStackTrace();
        }finally {
            if (response != null) {
                try {
                    response.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            if (httpClient != null) {
                try {
                    httpClient.close();
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        }
        return httpStatus;
    }


    /**
     * 发送PUT请求 <功能详细描述>
     *
     * @param url	路径
     * @param requestXML	消息体
     * @return
     * @throws UnsupportedEncodingException
     * @see [类、类#方法、类#成员]
     */
    public int putResourceXHxml(String url, String requestXML) {
        CloseableHttpClient httpClient = HttpClients.createDefault();
        CloseableHttpResponse response = null;
        int httpStatus = 500;
        try {
            InputStreamEntity reqEntity = new InputStreamEntity(new ByteArrayInputStream(requestXML.getBytes()));
            reqEntity.setContentType("application/xml");
            reqEntity.setContentEncoding("utf-8");
            reqEntity.setChunked(true);
            HttpUriRequest request = RequestBuilder.put()
                    .setUri(url)
                    .setEntity(reqEntity)
                    .setHeader("Content-Type", "application/xml")
                    .setHeader("requestType", CommonUtil.getConfValueByKey("XH_requestType"))
                    .setHeader("serviceID", CommonUtil.getConfValueByKey("XH_serviceId"))
                    .setHeader("secretKey", CommonUtil.getConfValueByKey("XH_secretKey"))
                    .build();
            LOG.info("请求星火服务续约接口，url：{}", url);
            response = httpClient.execute(request);
            httpStatus = response.getStatusLine().getStatusCode();
            LOG.info("响应码：{}", httpStatus);
        } catch (Exception e) {
            LOG.error("url：" + url + "请求出错！", e);
            e.printStackTrace();
        }finally {
            if (response != null) {
                try {
                    response.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            if (httpClient != null) {
                try {
                    httpClient.close();
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        }
        return httpStatus;
    }

    public int putResourceFirexml(String url, String requestXML) {
        CloseableHttpClient httpClient = HttpClients.createDefault();
        CloseableHttpResponse response = null;
        int httpStatus = 500;
        try {
            InputStreamEntity reqEntity = new InputStreamEntity(new ByteArrayInputStream(requestXML.getBytes()));
            reqEntity.setContentType("application/xml");
            reqEntity.setContentEncoding("utf-8");
            reqEntity.setChunked(true);
            HttpUriRequest request = RequestBuilder.put()
                    .setUri(url)
                    .setEntity(reqEntity)
                    .setHeader("Content-Type", "application/xml")
                    .setHeader("requestType", CommonUtil.getConfValueByKey("fire_requestType"))
                    .setHeader("serviceID", CommonUtil.getConfValueByKey("fire_serviceId"))
                    .setHeader("secretKey", CommonUtil.getConfValueByKey("fire_secretKey"))
                    .build();
            LOG.info("请求星火服务续约接口，url：{}", url);
            response = httpClient.execute(request);
            httpStatus = response.getStatusLine().getStatusCode();
            LOG.info("响应码：{}", httpStatus);
        } catch (Exception e) {
            LOG.error("url：" + url + "请求出错！", e);
            e.printStackTrace();
        }finally {
            if (response != null) {
                try {
                    response.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            if (httpClient != null) {
                try {
                    httpClient.close();
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        }
        return httpStatus;
    }

    public String postResourceFWKFPT(String url, List<NameValuePair> nameValuePairs, Map<String, Object> headerParamsMap) {
        HttpPost postMethod = new HttpPost(url);
        postMethod.setConfig(this.requestConfig);

        if(headerParamsMap != null && headerParamsMap.size() > 0){
            Set<String> keys = headerParamsMap.keySet();
            for (String key : keys) {
                postMethod.setHeader(key, headerParamsMap.get(key).toString());
            }
        }

        CloseableHttpResponse response = null;
        String content = null;
        try {
            postMethod.setEntity(new UrlEncodedFormEntity(nameValuePairs, "utf-8"));
            response = this.httpClient.execute(postMethod);
            int httpStatus = response.getStatusLine().getStatusCode();
            if (httpStatus != HttpStatus.SC_OK) {
                // 返回的是失败响应
                LOG.error(
                        String.format("Failed to execute post method '%s', caused by: it's response statuscode is %d!",
                                url, httpStatus));
            }
            HttpEntity entity = response.getEntity();
            if (entity != null) {
                content = EntityUtils.toString(entity, "UTF-8");
                EntityUtils.consume(entity);
            }
        } catch (Exception e) {
            LOG.error("url：" + url + "请求出错！", e);
            e.printStackTrace();
        } finally {
            if (response != null) {
                try {
                    response.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            postMethod.releaseConnection();
        }
        return content;
    }



    /**
     * 发送GET请求 <功能详细描述>
     *
     * @param url	get请求的url
     * @return
     * @see [类、类#方法、类#成员]
     */
    public String sendGetRequst(String url, Map<String, Object> headerParamsMap) {
        HttpGet httpGet = new HttpGet(url);
        if(headerParamsMap != null && headerParamsMap.size() > 0){
            Set<String> keys = headerParamsMap.keySet();
            for (String key : keys) {
                httpGet.setHeader(key, headerParamsMap.get(key).toString());
            }
        }
        CloseableHttpResponse response = null;
        String content = null;
        // 执行get请求
        try {
            response = this.httpClient.execute(httpGet);
            HttpEntity entity = response.getEntity();
            // 打印响应状态
            int httpStatus = response.getStatusLine().getStatusCode();
            if (httpStatus != HttpStatus.SC_OK) {
                // 返回的是失败响应
                LOG.error(
                        String.format("Failed to execute post method '%s', caused by: it's response statuscode is %d!",
                                url, httpStatus));
            }
            if (null != entity) {
                // 打印响应内容长度
                // 打印响应内容
                content = EntityUtils.toString(entity);
                EntityUtils.consume(entity); // 关闭资源
            }
        } catch (IOException e) {
            LOG.error("url：" + url + "请求出错！", e);
            e.printStackTrace();
        } finally {
            if (response != null) {
                try {
                    response.close();
                } catch (IOException e) {
                    e.printStackTrace();
                    LOG.error("", e);
                }
            }
            httpGet.releaseConnection();
        }
        return content;
    }

}
