package com.hhwy.qbeqsjy.controller;

import com.hhwy.qbeqsjy.common.Constants;
import com.hhwy.qbeqsjy.domain.AjaxResult;
import com.hhwy.qbeqsjy.dto.BusinessDTO;
import com.hhwy.qbeqsjy.service.CtrlService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;


@Slf4j
@RestController
@RequestMapping("/api/v1/ctrl")
public class CtrlController {

    @Value("${clue.type.list}")
    private String accountTypeList;
    @Value("${sys.id.list}")
    private String sysIdList;
    @Autowired
    private CtrlService ctrlService;


    /**
     * 下发布控任务
     * @param businessDTO
     * @return
     */
    @PostMapping("/submitCtrlTask")
    public AjaxResult submitCtrlTask(@RequestBody BusinessDTO businessDTO){
        String taskId = businessDTO.getTaskId();
        String account = businessDTO.getAccount();
        String accountType = businessDTO.getAccountType();
        String sysId = businessDTO.getSysId();
        if(StringUtils.isBlank(taskId) || StringUtils.isBlank(account) || StringUtils.isBlank(accountType) || StringUtils.isBlank(sysId)){
            return AjaxResult.error(Constants.PARAMS_EMPTY_MSG);
        }
        if(!accountTypeList.contains(accountType) || !sysIdList.contains(sysId)){
            return AjaxResult.error(Constants.PARAMS_ILLEGAL_MSG);
        }
        return ctrlService.ctrl(businessDTO);
    }


    /**
     * 停止布控任务
     * @param businessDTO
     * @return
     */
    @PostMapping("/stopCtrlTask")
    public AjaxResult stopCtrlTask(@RequestBody BusinessDTO businessDTO){
        String taskId = businessDTO.getTaskId();
        String account = businessDTO.getAccount();
        String accountType = businessDTO.getAccountType();
        String sysId = businessDTO.getSysId();
        if(StringUtils.isBlank(taskId) || StringUtils.isBlank(account) || StringUtils.isBlank(accountType) || StringUtils.isBlank(sysId)){
            return AjaxResult.error(Constants.PARAMS_EMPTY_MSG);
        }
        if(!accountTypeList.contains(accountType) || !sysIdList.contains(sysId)){
            return AjaxResult.error(Constants.PARAMS_ILLEGAL_MSG);
        }
        return ctrlService.stopCtrl(businessDTO);
    }


}
