package com.hhwy.qbeqsjy.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.hhwy.qbeqsjy.common.CommonUtil;
import com.hhwy.qbeqsjy.common.Constants;
import com.hhwy.qbeqsjy.common.HttpClient;
import com.hhwy.qbeqsjy.common.TimeTool;
import com.hhwy.qbeqsjy.domain.AjaxResult;
import com.hhwy.qbeqsjy.dto.BusinessDTO;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import java.util.HashMap;
import java.util.Map;

@Slf4j
@Service
public class CtrlService {

    @Value("${from.areacode}")
    private String fromAreaCode;
    @Value("${to.areacode}")
    private String toAreaCode;
    @Value("${to.sys}")
    private String toSys;
    @Value("${end.ctrl.time}")
    private String defaultEndCtrlTime;
    @Value("${serv.ver}")
    private String servVer;
    @Value("${sysuserid}")
    private String sysUserId;
    @Value("${uniCtrl.add.url}")
    private String uniCtrlAddUrl;
    @Value("${uniCtrl.del.url}")
    private String uniCtrlDelUrl;
    @Value("${uniCtrl.query.url}")
    private String uniCtrlQueryUrl;
    private HttpClient httpClient = new HttpClient();


    /**
     * 调用烽火【统一布控】接口，下发布控任务
     * @param businessDTO
     * @return
     */
    public AjaxResult ctrl(BusinessDTO businessDTO){
        String taskId = businessDTO.getTaskId();
        String account = businessDTO.getAccount();
        String accountType = businessDTO.getAccountType();
        String clueIdSrc = fromAreaCode.concat(taskId).concat("_").concat(accountType).concat(account);
        try{
            // 1.调用布控接口，下发布控任务
            JSONObject resJSONObj = doCtrl(httpClient, businessDTO);
            String statusCode = resJSONObj.getString("statuscode");
            String message = resJSONObj.getString("message");
            if (Constants.CTRL_PROCESS_SUCESS.equals(statusCode)) {
                // 2.布控请求处理成功（不代表实际布控成功，需查询线索布控状态信息）
                String clueId = resJSONObj.getString("clue_id");
                log.info("【布控请求处理成功】 clueId：{}", clueId);
                // 3.查询线索实际布控状态
                log.info("【查询线索布控状态】 clueId：{}", clueId);
                String status = queryCtrlStatus(httpClient, businessDTO);
                if(Constants.CTRL_SUCCESS.equals(status)){
                    log.info("【线索布控状态: 成功】 clueId：{}", clueId);
                    Map<String, Object> resultMap = new HashMap<>();
                    resultMap.put("clueId", clueId);
                    return AjaxResult.success(resultMap);
                }else if(Constants.CTRL_TODO.equals(status)){
                    log.info("【线索布控状态: 待布控】 clueId：{}", clueId);
                    return AjaxResult.success(Constants.CTRL_TODO_MSG);
                }else {
                    log.info("【线索布控状态: 失败】 clueId: {}, 响应码：{}", clueId, status);
                    return AjaxResult.error(Constants.CTRL_FAIL_MSG);
                }
            } else if (Constants.CTRL_EXIST.equals(statusCode)){
                log.error("【线索已经在控，无需再次发起布控】 clueId：{}", clueIdSrc);
                return AjaxResult.success(message);
            } else {
                // 如果布控请求处理失败，则直接返回异常信息，无需查询线索布控状态
                log.error("【线索布控请求处理失败】 clueId：{}, 响应码：{}, 异常信息：{}", clueIdSrc, statusCode, message);
                return AjaxResult.error(message);
            }
        } catch (Exception e){
            log.error("【下发布控任务发生异常】 异常信息：", e);
        }
        return AjaxResult.error(Constants.SERVER_ERROR_MSG);
    }


    /**
     * 调用烽火【统一布控】接口，下发停控任务
     * @param businessDTO
     * @return
     */
    public AjaxResult stopCtrl(BusinessDTO businessDTO){
        String taskId = businessDTO.getTaskId();
        String account = businessDTO.getAccount();
        String accountType = businessDTO.getAccountType();
        String clueIdSrc = fromAreaCode.concat(taskId).concat("_").concat(accountType).concat(account);
        try{
            // 调用停控接口
            JSONObject responseJSONObj = doStopCtrl(httpClient, businessDTO);
            String statusCode = responseJSONObj.getString("statuscode");
            String message = responseJSONObj.getString("message");
            if (Constants.CTRL_PROCESS_SUCESS.equals(statusCode)) {
                String clueId = responseJSONObj.getString("clue_id");
                log.info("【线索停控请求处理成功】 clueId：{}", clueId);
                log.info("【查询线索停控状态】 clueId：{}", clueId);
                String status = queryCtrlStatus(httpClient, businessDTO);
                if(Constants.STOP_CTRL_SUCCESS.equals(status)){
                    log.info("【线索停控状态: 成功】 clueId：{}", clueId);
                    Map<String, Object> resultMap = new HashMap<>();
                    resultMap.put("clueId", clueId);
                    return AjaxResult.success(resultMap);
                }else if(Constants.STOP_CTRL_TODO.equals(status)){
                    log.info("【线索停控状态: 待停控】 clueId: {}", clueId);
                    return AjaxResult.success(Constants.STOP_CTRL_TODO_MSG);
                }else {
                    log.info("【线索停控: 失败】 clueId: {}, 响应码：{}", clueId, status);
                    return AjaxResult.error(Constants.STOP_CTRL_FAIL_MSG);
                }
            } else if (Constants.STOP_CTRL_EXIST.equals(statusCode)){
                log.error("【线索已经停控，无需再次发起停控】 clueId：{}", clueIdSrc);
                return AjaxResult.success(message);

            } else {
                // 如果停控请求处理失败，则直接返回异常信息，无需查询线索停控状态
                log.error("【线索停控请求处理失败】 clueId：{}, 响应码：{}, 异常信息：{}", clueIdSrc, statusCode, message);
                return AjaxResult.error(message);
            }
        } catch (Exception e){
            log.error("【下发停控任务发生异常】 异常信息：", e);
        }
        return AjaxResult.error(Constants.SERVER_ERROR_MSG);
    }


    /**
     * 调用烽火【统一布控】接口, 下发布控请求 - 布控失败时，重试3次
     * @param httpClient
     * @param businessDTO
     * @return
     */
    public JSONObject doCtrl(HttpClient httpClient, BusinessDTO businessDTO) {
        // 构建请求参数
        String resourceIdList = businessDTO.getResourceIdList();
        JSONObject params = new JSONObject();
        // 将配置文件中的数据资源业务ID转换为烽火盘古平台数据资源编码
        if(StringUtils.isNotBlank(resourceIdList) && resourceIdList.split(",").length > 0){
            String[] resourceIdArray = resourceIdList.split(",");
            String[] resourceCodeArray = new String[resourceIdArray.length];
            for (int i = 0; i < resourceIdArray.length; i++) {
                String resourceId = resourceIdArray[i];
                String resourceCode = CommonUtil.getConfValueByKey(resourceId);
                if(StringUtils.isNotBlank(resourceCode)){
                    resourceCodeArray[i] = resourceCode;
                }
            }
            params.put("resid_list", resourceCodeArray);
        } else {
            throw new RuntimeException("待布控数据资源列表为空，布控任务下发失败");
        }
        String taskId = businessDTO.getTaskId();
        String account = businessDTO.getAccount();
        String accountType = businessDTO.getAccountType();
        String clueId = fromAreaCode.concat(taskId).concat("_").concat(accountType).concat(account);
        String sysId = businessDTO.getSysId();
        String endCtrlTime = businessDTO.getEndCtrlTime();

        params.put("clue_id", clueId);
        params.put("from_areacode", fromAreaCode);
        params.put("to_areacode", JSONArray.parseArray(toAreaCode));
        params.put("from_sys", sysId);
        params.put("to_sys", JSONArray.parseArray(toSys));
        params.put("sysuserid", sysUserId);
        params.put("serv_ver", servVer);
        params.put("property_type", accountType);
        params.put("property_value", account);
        if(StringUtils.isNotBlank(endCtrlTime)){
            params.put("end_ctrl_time", TimeTool.dateStringToTimeStamp(endCtrlTime));
        } else {
            params.put("end_ctrl_time", defaultEndCtrlTime);
        }
        JSONObject response = new JSONObject();
        int n = Constants.NUM_0;
        while (n <= Constants.NUM_3) {
            String responseStr = httpClient.postResource(uniCtrlAddUrl, params.toJSONString(), null);
            response = JSONObject.parseObject(responseStr);
//            log.info(response.toJSONString());
            int statusCode = response.getInteger("statuscode");
            if (statusCode != Constants.REQ_RES_CATALOG_FAIL && statusCode != Constants.REQ_OTHER_EXCEP) {
                break;
            }
            n++;
            log.error("处理布控请求发生异常，正在重试，布控线索ID：{}, 响应信息：{}", clueId, response);
            CommonUtil.threadSleep(Constants.NUMBER_3000L);
        }
        return response;
    }


    /**
     * 调用烽火【统一布控】接口, 下发停控请求 - 布控失败时，重试3次
     * @param httpClient
     * @param businessDTO
     * @return
     */
    public JSONObject doStopCtrl(HttpClient httpClient, BusinessDTO businessDTO) {
        JSONObject params = new JSONObject();
        String taskId = businessDTO.getTaskId();
        String account = businessDTO.getAccount();
        String accountType = businessDTO.getAccountType();
        String sysId = businessDTO.getSysId();
        String clueId = fromAreaCode.concat(taskId).concat("_").concat(accountType).concat(account);
        params.put("clue_id", clueId);
        params.put("from_areacode", fromAreaCode);
        params.put("from_sys", sysId);
        params.put("serv_ver", servVer);
        params.put("sysuserid", sysUserId);
        JSONObject response = new JSONObject();
        int n = Constants.NUM_0;
        while (n <= Constants.NUM_3) {
            String responseStr = httpClient.postResource(uniCtrlDelUrl, params.toJSONString(), null);
            response = JSONObject.parseObject(responseStr);
            int statusCode = response.getInteger("statuscode");
            if (statusCode != Constants.REQ_RES_CATALOG_FAIL && statusCode != Constants.REQ_OTHER_EXCEP) {
                break;
            }
            n++;
            log.error("处理停控请求发生异常, 正在重试, 停控线索ID: {}, 响应信息: {}", clueId, response);
            CommonUtil.threadSleep(Constants.NUMBER_2000L);
        }
        return response;
    }


    /**
     * 调用烽火【统一布控】接口, 查询线索布/停控状态 - 查询失败时，重试3次
     * @param httpClient
     * @param businessDTO 需要查询状态的布控内容
     * @return String 布/停控状态
     */
    public String queryCtrlStatus(HttpClient httpClient, BusinessDTO businessDTO) {
        JSONObject params = new JSONObject();
        String taskId = businessDTO.getTaskId();
        String account = businessDTO.getAccount();
        String accountType = businessDTO.getAccountType();
        String sysId = businessDTO.getSysId();
        String clueId = fromAreaCode.concat(taskId).concat("_").concat(accountType).concat(account);
        params.put("clue_id", clueId);
        params.put("from_areacode", fromAreaCode);
        params.put("from_sys", sysId);
        params.put("serv_ver", servVer);
        int n = 0;
        int m = 0;
        String status = "0";
        while (n <= Constants.NUM_3) {
            String responseStr = httpClient.postResource(uniCtrlQueryUrl, params.toJSONString(), null);
            JSONObject response = JSONObject.parseObject(responseStr);
            String statusCode = response.getString("statuscode"); // 查询响应状态码
            status = response.getString("status");// 布控状态码
            // 根据查询结果做不同处理
            if (Constants.QUERY_OTHER_EXCEP.equals(statusCode)) { //其他异常，间隔1s，重试3次
                n++;
                log.error("查询布/停控状态异常, 响应状态:2999, 重试第{}次, 布/停控线索ID:{}", n, clueId);
            } else if (Constants.QUERY_SUC.equals(statusCode) && (Constants.CTRL_TODO.equals(status)  || Constants.CTRL_ING.equals(status) || Constants.STOP_CTRL_TODO.equals(status) || Constants.STOP_CTRL_ING.equals(status))) {
                m++;
                log.info("查询布/停控状态中, 响应状态:{}, 布/停控状态:{}, 重试第{}次, 布/停控线索ID:{}", statusCode, status, m, clueId);
                if (m > Constants.NUM_10) {
                    break;
                }
            } else { //其他情况直接返回查询结果
                log.info("查询布/停控状态完成, 响应状态:{}, 布/停控状态:{}, 布/停控线索ID:{}", statusCode, status, clueId);
                break;
            }
            CommonUtil.threadSleep(Constants.NUMBER_2000L);
        }
        return status;
    }


}
